/**********
Copyright 2017 Intel Corporation.

The source code, information and material ('Material') contained herein is owned
 by Intel Corporation or its suppliers or licensors, and title to such Material 
 remains with Intel Corporation or its suppliers or licensors. The Material contains
 proprietary information of Intel or its suppliers and licensors. The Material is 
 protected by worldwide copyright laws and treaty provisions. No part of the Material
 may be used, copied, reproduced, modified, published, uploaded, posted, transmitted, 
 distributed or disclosed in any way without Intel's prior express written permission. 
 No license under any patent, copyright or other intellectual property rights in 
 the Material is granted to or conferred upon you, either expressly, by implication, 
 inducement, estoppel or otherwise. Any license under such intellectual property 
 rights must be express and approved by Intel in writing.

Include any supplier copyright notices as supplier requires Intel to use.

Include supplier trademarks or logos as supplier requires Intel to use, preceded 
by an asterisk. An asterisked footnote can be added as follows: *Third Party 
trademarks are the property of their respective owners.

Unless otherwise agreed by Intel in writing, you may not remove or alter this notice 
or any other notice embedded in Materials by Intel or Intel's suppliers or licensors 
in any way.
**********/
#pragma once

#include "Particles.h"

namespace Particle { namespace Actions {

// Gravity updates velocity from a constant acceleration vector
struct Gravity : Action {
    vector g;
    Gravity(vector const&);
    void operator()(time, iterator) override;
};

// Move updates position based on velocity
struct Move : Action {
    void operator()(time, iterator) override;
};

// Bounce causes particles to bounce off a domain geometry
template<typename Domain>
struct Bounce : Action {
    Domain domain;
    vector::value_type oneMinusFriction, resilience, cutoff;
    Bounce(vector::value_type friction, vector::value_type resilience, vector::value_type cutoff, Domain&&);
    void operator()(time, iterator) override;
};

// SinkPosition kills particles not inside/outside a domain geometry
template<typename Domain>
struct SinkPosition : Action {
    Domain domain;
    bool kill_inside;
    SinkPosition(bool, Domain&&);
    void operator()(time, iterator) override;
};

} }
