/**********
Copyright 2017 Intel Corporation.

The source code, information and material ('Material') contained herein is owned
 by Intel Corporation or its suppliers or licensors, and title to such Material 
 remains with Intel Corporation or its suppliers or licensors. The Material contains
 proprietary information of Intel or its suppliers and licensors. The Material is 
 protected by worldwide copyright laws and treaty provisions. No part of the Material
 may be used, copied, reproduced, modified, published, uploaded, posted, transmitted, 
 distributed or disclosed in any way without Intel's prior express written permission. 
 No license under any patent, copyright or other intellectual property rights in 
 the Material is granted to or conferred upon you, either expressly, by implication, 
 inducement, estoppel or otherwise. Any license under such intellectual property 
 rights must be express and approved by Intel in writing.

Include any supplier copyright notices as supplier requires Intel to use.

Include supplier trademarks or logos as supplier requires Intel to use, preceded 
by an asterisk. An asterisked footnote can be added as follows: *Third Party 
trademarks are the property of their respective owners.

Unless otherwise agreed by Intel in writing, you may not remove or alter this notice 
or any other notice embedded in Materials by Intel or Intel's suppliers or licensors 
in any way.
**********/
#pragma once

#include "Particles.h"

namespace Particle { namespace Domains {

// Genearal definition of a domain
struct Domain 
{
    bool isIn(vector const&) const;     // is this vector in this domain?
    template<typename T> void emit(T&); // emit some value from this domain.
};

// A Constant domain always returns a single value
struct Constant 
{
    vector::value_type k;                   
    Constant(vector::value_type const&);
    void emit(vector::value_type&);
};

// A Point domain always returns a single point
struct Point
{
    vector p;
    Point(vector const&);
    void emit(vector&);
};

// A Color domain 
struct Color 
{
    color c1, c2;                       // end-points in color-space
    Color(color const&, color const&);  // construct
    void emit(color&);                  // emit randomly between end-points
};

// A Line domain is a line segment between two points
struct Line
{
    vector p1, p2;
    Line(vector const&, vector const&);
    void emit(vector&);
};

// A Triangle domain 
struct Triangle
{
    vector p, u, v, n; vector::value_type d;
    Triangle(vector const&, vector const&, vector const&);
    void emit(vector&);
};

// A Plane domain
struct Plane
{
    vector p, u, v, n; vector::value_type d;
    Plane(vector const&, vector const&, vector const&);
    bool isIn(vector const&) const;
    void emit(vector&);
};

// A Sphere domain
struct Sphere
{
    vector p; vector::value_type r1, r2;
    Sphere(vector const&, vector::value_type r1, vector::value_type r2);
    bool isIn(vector const&) const;
    void emit(vector&);
};

// A Box domain
struct Box
{
    vector pmin, pmax;
    Box(vector const&, vector const&);
    bool isIn(vector const&) const;
    void emit(vector&);
};

// A Disc domain
struct Disc
{
    vector p, n; vector::value_type r1, r2, d;
    Disc(vector const&, vector const&, vector::value_type r1, vector::value_type r2);
    void emit(vector&);
};

// A Cylinder domain
struct Cylinder 
{
    vector p1,p2,u,v; vector::value_type r1, r2;
    Cylinder(vector const&, vector const&, vector::value_type r1, vector::value_type r2);
    void emit(vector&);
};

}}

