/**********
Copyright 2017 Intel Corporation.

The source code, information and material ('Material') contained herein is owned
 by Intel Corporation or its suppliers or licensors, and title to such Material 
 remains with Intel Corporation or its suppliers or licensors. The Material contains
 proprietary information of Intel or its suppliers and licensors. The Material is 
 protected by worldwide copyright laws and treaty provisions. No part of the Material
 may be used, copied, reproduced, modified, published, uploaded, posted, transmitted, 
 distributed or disclosed in any way without Intel's prior express written permission. 
 No license under any patent, copyright or other intellectual property rights in 
 the Material is granted to or conferred upon you, either expressly, by implication, 
 inducement, estoppel or otherwise. Any license under such intellectual property 
 rights must be express and approved by Intel in writing.

Include any supplier copyright notices as supplier requires Intel to use.

Include supplier trademarks or logos as supplier requires Intel to use, preceded 
by an asterisk. An asterisked footnote can be added as follows: *Third Party 
trademarks are the property of their respective owners.

Unless otherwise agreed by Intel in writing, you may not remove or alter this notice 
or any other notice embedded in Materials by Intel or Intel's suppliers or licensors 
in any way.
**********/
#include <windows.h>

#include "Particles.h"

#include "Initializers.h"
#include "Renderers.h"
#include "Sources.h"
#include "Actions.h"
#include "Domains.h"

#include <chrono>

// std::chrono demands this if you want to use the "min" literal operator
using namespace std::chrono;

#define VISUAL 1
#define OCULUS 0

int main(int argc, char const* argv[])
{
    // Set our priority high to get consistant performance
    if (!SetPriorityClass(GetCurrentProcess(), HIGH_PRIORITY_CLASS))
        fprintf(stderr, "YIKES!  Can't set HIGH_PRIORITY_CLASS -- %08X\n", GetLastError());

    // Create the particle system
    Particle::System particles{

        // Source -- initializers, one for each field in a particle
#if VISUAL
        new Particle::Sources::Flow{ 10240*3/2, {
#else
        new Particle::Sources::Max{  {
#endif
            new Particle::Initializers::Age     <Particle::Domains::Constant> { 0.f },
            new Particle::Initializers::Position<Particle::Domains::Point   >{{ {0,0,0.15} }},
            new Particle::Initializers::Velocity<Particle::Domains::Cylinder>{{ {0.0,0,0.85}, {0.0,0,1.97}, 0.21f, 0.19f }},
            new Particle::Initializers::Color   <Particle::Domains::Color   >{{ {255,0,0,255}, {0,0,0,255} }},
        }},

        { // Actions
            new Particle::Actions::Gravity{{0,0,-1.8}},
            new Particle::Actions::Bounce<Particle::Domains::Disc>{0.f, 0.35f, 0.f, {{0,0,0}, {0,0,1}, 1.5, 0}},
            new Particle::Actions::Move{},
            new Particle::Actions::SinkPosition<Particle::Domains::Plane>{false, {{0,0,-60},{1,0,-60},{0,1,-60}}},
        },

        // Renderer
#if VISUAL
#if OCULUS
        new Particle::Renderers::Oculus{}
#else
        new Particle::Renderers::OpenGL{}
#endif
#else
        new Particle::Renderers::Null{}
#endif
    };

    // Run the particle system
    auto start = high_resolution_clock::now(), last = start, now = start; do 
    {
        auto dt = duration<Particle::time>(now-last).count();
        if (particles.frame(dt)) break;
        last = now; now = high_resolution_clock::now();

    } while (now-start < 3min || true);

    // Exit
    return 0;
}
